<?php

namespace WPMC\Controller;

use GuzzleHttp\Psr7\ServerRequest;
use Psr\Http\Message\ResponseInterface;
use WPMC\HttpHandler\ResponseEmitter;
use WPMC\Plugin;
use WPMC\Service\MyChannelsApiService;

class FrontendController
{


    public static function setup() {

        add_action('init', [self::class,'registerQueryVar']);
        add_action('init', [self::class,'registerRewriteRule']);
        add_action('parse_request', [self::class,'registerFrontendHandling']);
        add_filter('redirect_canonical', [self::class,'disableMountpointCanonicalRedirect'], 10, 2);

    }


    public static function registerQueryVar() {

        add_rewrite_tag('%'.Plugin::FRONTEND_QUERY_VAR.'%', '([^&]+)');

    }

    public static function registerRewriteRule() {

        $mountpoint = Plugin::getSetting(Plugin::SETTING_MOUNTPOINT);

        add_rewrite_rule(
            Plugin::getRewriteRuleRegex($mountpoint),
            Plugin::FRONTEND_REWRITE_QUERY,
            'top'
        );

    }


    public static function disableMountpointCanonicalRedirect($redirectUrl = null, $requestedUrl = null) {

        $mountpoint = Plugin::getSetting(Plugin::SETTING_MOUNTPOINT);

        $urlPath = parse_url($requestedUrl, PHP_URL_PATH);
        $regPattern = '#^/'.$mountpoint.'#';
        if ( preg_match($regPattern,$urlPath) ) {
            return false;
        }
        return $redirectUrl;

    }


    public static function registerFrontendHandling(\WP $wp) {

        if(array_key_exists(Plugin::FRONTEND_QUERY_VAR, $wp->query_vars)) {

            if(! Plugin::connectionIsHttps()) {
                self::send404Response();
                return;
            }

            $requestedUri = $wp->query_vars[Plugin::FRONTEND_QUERY_VAR];
            $isSanityCheckerRequest = (strpos($requestedUri, 'sanity') === 0);

            if ($isSanityCheckerRequest && !current_user_can(Plugin::REQUIRED_ADMIN_CAPABILITY)) {
                //Only allowed for admins
                if (!current_user_can(Plugin::REQUIRED_ADMIN_CAPABILITY)) {
                    self::send404Response();
                    return;
                }
            }

            $api = self::getMyChannelsApiService();

            $serverRequest = ServerRequest::fromGlobals();
            $userId = (is_user_logged_in()) ? wp_get_current_user()->ID : null;

            $apiResponse = $api->propagateServerRequest($serverRequest, $userId);

            $isAjaxRequest = (strtolower($serverRequest->getHeaderLine('X-Requested-With')) === 'xmlhttprequest');
            $isStitcherResponse = (strpos($apiResponse->getHeaderLine('Content-Type'), 'stitcher') !== false);

            $responseCode = $apiResponse->getStatusCode();

            if($isAjaxRequest) {
                //We emit the API response 1:1 to our client
                self::sendResponse($apiResponse);
            }
            else {

                if($responseCode === 200 && $isStitcherResponse) {
                    self::sendStitchedResponse($apiResponse);
                }
                elseif($responseCode === 404) {
                    self::send404Response();
                }
                else {
                    //We emit the API response 1:1 to our client, e.g. 301, 302 or other responses
                    self::sendResponse($apiResponse);
                }
            }

        }


    }


    private static function sendResponse(ResponseInterface $response) {

        $responseEmitter = new ResponseEmitter();
        $responseEmitter->emit($response);
        die;

    }


    private static function send404Response() {

        add_filter('template_include', function(){
            /** @var \WP_Query $wp_query */
            global $wp_query;
            $wp_query->set_404();
            status_header(404);
            nocache_headers();
            return get_query_template('404');
        });

    }

    private static function sendStitchedResponse(ResponseInterface $apiResponse) {

        $responseBody = $apiResponse->getBody()->getContents();
        $responseData = json_decode($responseBody, true);

        if(json_last_error()!==JSON_ERROR_NONE) {
            throw new \RuntimeException('Invalid json response');
        }

        if(!isset($responseData['html']) || ! is_string($responseData['html'])) {
            throw new \RuntimeException('html not present in json response');
        }

        //Add stylesheets
        if(isset($responseData['stylesheets']) && is_array($responseData['stylesheets'])) {
            foreach ($responseData['stylesheets'] as $i => $stylesheetUrl) {
                wp_enqueue_style('wpmc-style-' . $i, $stylesheetUrl, [], null);
            }
        }

        //Add scripts
        if(isset($responseData['javascripts']) && is_array($responseData['javascripts'])) {
            foreach ($responseData['javascripts'] as $i => $scriptUrl) {
                wp_enqueue_script('wpmc-script-' . $i, $scriptUrl, [], null);
            }
        }


        if(isset($responseData['meta']) && is_array($responseData['meta'])) {

            $metaProperties = $responseData['meta'];

            //Add meta tags
            add_action('wp_head', function () use ($metaProperties) {

                if (isset($metaProperties['names']) && is_array($metaProperties['names'])) {
                    foreach ($metaProperties['names'] as $prop) {
                        if(isset($prop['key']) && isset($prop['value']) && is_string($prop['key']) && is_string($prop['value'])) {
                            echo '<meta name="' . esc_attr($prop['key']) . '" content="' . esc_attr($prop['value']) . '">';
                        }
                    }
                }

                if (isset($metaProperties['properties']) && is_array($metaProperties['properties'])) {
                    foreach ($metaProperties['properties'] as $prop) {
                        if(isset($prop['key']) && isset($prop['value']) && is_string($prop['key']) && is_string($prop['value'])) {
                            echo '<meta property="' . esc_attr($prop['key']) . '" content="' . esc_attr($prop['value']) . '">';
                        }
                    }
                }
            });

            //Set page title
            if(isset($metaProperties['title']) && is_string($metaProperties['title'])) {
                if(current_theme_supports('title-tag')){
                    add_filter('document_title_parts', function ($titleParts) use ($metaProperties) {
                        $titleParts['title'] = $metaProperties['title'];
                        return $titleParts;
                    }, 12 ,1);
                }
                else {
                    add_filter('wp_title', function () use ($metaProperties) {
                        return $metaProperties['title'];
                    }, 12, 0);
                }
            }
        }

        //Set html body
        add_action(Plugin::TAG_ACTION_FRONTEND_OUTPUT_BODY, function() use ($responseData) {
            echo $responseData['html'];
        }, 10, 1);

        //Set template
        add_filter('template_include', function() {

            $template = WPMC_PLUGIN_PATH . '/templates/frontend.php';
            return apply_filters(Plugin::TAG_FILTER_FRONTEND_TEMPLATE_PATH, $template);

        });

    }


    /**
     * @return MyChannelsApiService
     */
    public static function getMyChannelsApiService(){

        $apiHost = Plugin::getSetting(Plugin::SETTING_HOSTNAME);
        $apiUsername = Plugin::getSetting(Plugin::SETTING_USERNAME);
        $apiPassword = Plugin::getSetting(Plugin::SETTING_PASSWORD);
        $siteIdentifier = Plugin::getSetting(Plugin::SETTING_SITE_IDENTIFIER);
        $logFile = (Plugin::getSetting(Plugin::SETTING_LOGGING, false)) ? Plugin::getLogFilePath() : null;

        return new MyChannelsApiService($apiHost, $apiUsername, $apiPassword, $siteIdentifier, $logFile);

    }


}
