<?php

namespace WPMC\Service;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\BadResponseException;
use GuzzleHttp\Psr7\Request;
use GuzzleHttp\Psr7\Response;
use GuzzleHttp\Psr7\Uri;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;


class MyChannelsApiService
{

    const STITCHER_CONTENT_TYPE = 'application/vnd.mychannels.stitcher+json';

    private $apiHost;
    private $apiUsername;
    private $apiPassword;

    private $siteIdentifier;

    private $client;

    protected static $instance;

    private $logFilePath;
    private $logFileStream;

    /**
     * @param $apiHost
     * @param $apiUsername
     * @param $apiPassword
     * @param $siteIdentifier
     * @param resource $logFilePath
     */
    public function __construct($apiHost, $apiUsername, $apiPassword, $siteIdentifier, $logFilePath = null)
    {
        $this->apiHost = $apiHost;
        $this->apiUsername = $apiUsername;
        $this->apiPassword = $apiPassword;
        $this->siteIdentifier = $siteIdentifier;
        $this->logFilePath = $logFilePath;
        if($logFilePath) {
            $this->logFileStream = fopen($logFilePath, 'a');
        }
    }


    /**
     * @param $mountPoint
     * @param ServerRequestInterface $serverRequest
     * @return mixed|ResponseInterface
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function testSettings($mountPoint, ServerRequestInterface $serverRequest) {

        $uri = (new Uri())->withHost($this->apiHost)->withPath($mountPoint)->withScheme('https');

        $requestOptions = [];

        $request = (new Request('GET', $uri))
            ->withHeader('Forwarded', self::getForwardedHeader($serverRequest->getServerParams()))
            ->withHeader('Accept', self::STITCHER_CONTENT_TYPE);

         /** @noinspection PhpParamsInspection */
         $response = $this->getClient()->send($request, $requestOptions);
         return $response;

    }


    /**
     * @param ServerRequestInterface $serverRequest
     * @param int|null $userId
     * @return ResponseInterface
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function propagateServerRequest(ServerRequestInterface $serverRequest, $userId = null) {

        $requestOptions = [];

        $serverParams = $serverRequest->getServerParams();

        $request = $serverRequest
            ->withUri($serverRequest->getUri()->withHost($this->apiHost))
            ->withHeader('Forwarded', self::getForwardedHeader($serverParams))
        ;

        if($userId !== null){
            $request = $request->withHeader('External-User-Id', $userId)
                               ->withHeader('External-User-Source', $this->siteIdentifier);
        }

        $isAjax = (strtolower($serverRequest->getHeaderLine('X-Requested-With')) === 'xmlhttprequest');
        if(!$isAjax) {
            $request = $request->withHeader('Accept', self::STITCHER_CONTENT_TYPE);
        }

        //Form Params fix
        if(strpos( $request->getHeaderLine('Content-Type'),'multipart')!==false) {
            $request = $request->withoutHeader('Content-Length')
                               ->withoutHeader('Content-Type');
            $requestOptions['form_params'] = $serverRequest->getParsedBody();
        }

        try {
            /** @noinspection PhpParamsInspection */
            $response = $this->getClient()->send($request, $requestOptions);
            return $response;
        }
        catch(BadResponseException $e){
            if($this->logFilePath) error_log((string)$e.PHP_EOL, 3, $this->logFilePath);

            $response = $e->getResponse();
            return $response;
        }
        catch(\RuntimeException $e){
            if($this->logFilePath) error_log((string)$e.PHP_EOL, 3, $this->logFilePath);
            return new Response(503, []); //503 = Service Unavailable
        }

    }



    /**
     * @return Client
     */
    private function getClient(){

        if($this->client === null) {

            $clientConfig = [
                'allow_redirects' => false,
                'http_errors' => false,
            ];

            if($this->apiUsername && $this->apiPassword) {
                $clientConfig['auth'] = [$this->apiUsername, $this->apiPassword];
            }

            if($this->logFileStream) {
                $clientConfig['debug'] = $this->logFileStream;
            }

            $this->client = new Client($clientConfig);
        }

        return $this->client;

    }


    /**
     * @param array $serverParams
     * @return string
     * @see https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Forwarded
     */
    private static function getForwardedHeader($serverParams) {

        $procotol = isset($serverParams['REQUEST_SCHEME']) ? $serverParams['REQUEST_SCHEME'] :
                    (isset($serverParams['HTTPS']) ? 'https' : 'http');

        $forwardedParams = [
            'by' => (isset($serverParams['SERVER_ADDR']) ? $serverParams['SERVER_ADDR'] : 'unknown'),
            'for' => (isset($serverParams['REMOTE_ADDR']) ? $serverParams['REMOTE_ADDR'] : 'unknown'),
            'host' => (isset($serverParams['HTTP_HOST']) ? $serverParams['HTTP_HOST'] : ''),
            'proto' => $procotol
        ];

        return self::buildCsvHeader($forwardedParams);

    }


    /**
     * @param array $params
     * @return string
     */
    private static function buildCsvHeader($params) {
        $components = [];
        foreach($params as $key=>$value){
            $components[] = "$key=$value";
        }
        return implode(';',$components);
    }



}
